# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"download_dataset": "00_core.ipynb",
         "display_large": "00_core.ipynb",
         "rf_feature_importance": "00_core.ipynb",
         "plot_feature_importance": "00_core.ipynb"}

modules = ["core.py"]

doc_url = "https://lewtun.github.io/hepml/"

git_url = "https://github.com/lewtun/hepml/tree/master/"

def custom_doc_links(name): return None
